==> GeneR_2.6.0.tar.gz: GeneR/src/masked.cc <==
/*! \file masked.cc
 *  \brief functions to look for masked positions. Masked by letter or
 *  by lower / upper case.
 *  \version 1
 *  \date Created       : 26/10/05 
 *  \date Last Modified : Time-stamp: <2006-02-13 11:36:49 lucas>
 *  \author O. Rogier and A. Lucas
 *  \note Licence: CeCIll
 */


#include <stdio.h>
#include <string>
#include "GeneR_glob.h"
#include "masked.h"

#define min( A , B )  ( ( A ) < ( B ) ? ( A ) : ( B ) )



extern "C"
{
  void mask (int * bufno,int * from, int * to, int * n, char ** lettre,int * err);
  void maskee(char ** file, int * debSeq, int * tailleSeq, int * from, 
	      int * to,int * tailleAlloc, int* fromSeq, int* toSeq,int * err);
  void mask_buffer(int * bufno,  int * from, int * to,int * tailleAlloc,  
		   int* fromSeq, int* toSeq,int * err);
  void mask_buffer_char(int * bufno,  int * from, int * to,int * tailleAlloc, 
			char ** caracter,  int* fromSeq, int* toSeq, 
			int * err);
}


/** \fn mask (int * bufno,int * from, int * to, char ** lettre,int * err)
 * \brief mask parts of a sequence with letter 
 * \author A. Lucas
 */
void mask (int * bufno,int * from, int * to, int * n,char ** lettre,int * err)
{
  char * seq;
  int i,j;
  seq = myGeneRbuffer(bufno[0],0);
  if(seq == NULL)
    {
      *err = -1;
      return;
    }

  GeneRfreeCompSeq(*bufno);


  for(i = 0; i< (*n); i++)
    {
      for(j=from[i];j<=to[i]; j++)
	seq[j-1] = lettre[0][0];

    }  

}

/** \fn maskee(char ** file, int * debSeq, int * tailleSeq, int * from, int * to,int * tailleAlloc, int * err)
 * \brief This function gives the masked sequences positions from a fasta file
 *  It returns : 1 if the vector of positions has been created
 *              0 if the allocation length is too small
 *             -1 if error (can't open the file) 
 * \author O. Orgier
 */
void maskee(char ** file, int * debSeq, int * tailleSeq, int * from, int * to,int * tailleAlloc,  int* fromSeq, int* toSeq,int * err)
{

  char * seq;
  FILE * fichier;

  
  fichier = fopen(*file,"r");
  if (fichier == NULL)
    {
      printf("File Name incorrect !\n"); 
      * err = -1;
      return;
    }
  seq = (char *) malloc ((* tailleSeq) * sizeof(char));
 


  *tailleSeq = masked::lecturefasta(fichier,seq,* debSeq,* tailleSeq)-1;

  if(*toSeq == 0)
    *toSeq = *tailleSeq;

  /*  * tailleSeq = strlen(seq)-1; */

  /*  seq[* tailleSeq] = '\0' ;*/
  /*printf("Sequence length : %ld \n",* tailleSeq);
    printf("Sequence :%s \n",seq); */

  * err = masked::codage(seq,tailleAlloc,from,to ,fromSeq, toSeq);
  /*if (*err == 0 ) {printf("Warning message : not all the sequence was masked.\n");}
    printf("Number of masked sequences: %d\n",*tailleAlloc);*/

  free(seq);
  return;
}


/** \brief This function gives the masked sequences positions
 * between pos begSeq and EndSeq. position are returned in vector
 * from and to.
 * It returns : 1 if the vector of positions has been created
 *              0 if the allocation length is too small
 *             -1 if error (can't open the file) 
 * \author A. Lucas and O. Rogier
 */
void mask_buffer(int * bufno,  int * from, int * to,int * tailleAlloc,
		 int * begSeq, int * endSeq,
		 int * err)
{

  char * seq;

  seq = myGeneRbuffer(bufno[0],0);
  if(seq == NULL)
    {
      *err = -1;
      return;
    }

  * err = masked::codage(seq,tailleAlloc,from,to,begSeq,endSeq);
  /*  if (*err == 0 ) {printf("Warning message : not all the sequence was masked.\n");}*/
  /*  printf("Number of masked sequences: %d\n",*tailleAlloc);*/
  
  
  return;
}



/** mask_buffer_char
 *
 *
 * \author A. Lucas and O. Rogier
 */
void mask_buffer_char(int * bufno,  int * from, int * to,int * tailleAlloc, 
		     char ** caracter, int* begSeq, int* endSeq, int * err)
{

  char * seq;


  seq = myGeneRbuffer(bufno[0],0);

  * err = masked::codage_char(seq,tailleAlloc,from,to,caracter,begSeq,endSeq);
  /*  if (*err == 0 ) {printf("Warning message : not all the sequence was masked.\n");}*/
  /*  printf("Number of masked sequences: %d\n",*tailleAlloc);*/
  
  
  return;
}


namespace masked{
  /** \fn lecturefasta (FILE * fichier,char * seq, int debSeq, int tailleSeq)
  * \brief Read a fasta file to a char*
   * \author A. Lucas and O. Rogier
   */ 
  int lecturefasta (FILE * fichier,char * seq, int debSeq, int tailleSeq)
  {
    int indice,longueur_cour,longueur_ligne,longueur_rest;
    char  tmp [512];

#ifndef __MINGW_H  
    if(fseek( fichier, debSeq, SEEK_SET))
      return(-1);
#else
    int i;
    for(i=0; i< debSeq; i++)
      if(fgetc(fichier) == EOF)
	return(-1);
#endif
    fgets(seq,tailleSeq,fichier);      
    indice =strlen(seq);
    seq[indice-1] = '\0'; 
    longueur_ligne = longueur_cour = indice-1;
    longueur_rest = tailleSeq - longueur_ligne;
  

    while(  fgets(tmp,min(512,longueur_rest),fichier) != NULL)
      {
	indice =strlen(tmp);
	tmp[indice-1] = '\0'; /* Delete \n */
	
	// memcpy: bien plus rapide que strcat 
	if(tmp[0] != '>')
	  memcpy(seq+longueur_cour,tmp,indice);
	longueur_cour += indice-1;
	longueur_rest -= indice-1;
	//strcat(seq,tmp);
      }

    /*  printf("seq: >%s<\n",seq); */

    fclose(fichier);
    return(longueur_cour);
  }


  /** codage 
   * \brief get positions when sequence change from lower to upper case 
   * between pos begSeq & endSeq
   * \author O. Rogier, modified by A. Lucas.
   */
  int codage(char * seq, int * tailleAlloc, int * from, int * to,
	     int * begSeq, int * endSeq)
    
  {
    int i,j,b,majuscule;

    j=0;
    b=1;
    majuscule = 1;

    for(i=begSeq[0]; i<endSeq[0] ; i++)
      {
	if(seq[i] > 'Z')
	  {
	    if(majuscule)
	      {
		if (j>= * tailleAlloc) {b = 0;break;}	      
		from[j] = i+1;
		majuscule = 0;
	      }
	  }
	else
	  if(!majuscule)
	    {
	      to[j] = i;
	      majuscule = 1;
	      j++;    
	    }
      }

    /* if last letters were in lower case */
    /*  printf("|%c|\n",seq[i-1]);
	printf(">%s<\n",seq); */
    if(seq[i-1] > 'Z') /* equivalent to if(majucule == 0)*/
      {
	to[j] = i;
	j++;
      }

    /* from = (int *) realloc(from, (j+1)* sizeof(int));
       to = (int *) realloc(to, (j+1)* sizeof(int)); */
  
    * tailleAlloc = j;
    return(b);
  }


  /** codage_char 
   * \brief get positions when sequence is a special char (by example N)
   * \author A. Lucas and O. Rogier 
   */
  int codage_char(char * seq, int * tailleAlloc, int * from, int * to,char ** caracter, int * fromSeq, int * endSeq)
    
  {
    int i,j,b,pascommececaracter;
    j=0;
    b=1;
    pascommececaracter = 1;
    
    for(i=fromSeq[0]; i < endSeq[0] ; i++)
      {
	if(seq[i] == (**caracter) )
	  {
	    if(pascommececaracter)
	      {
		if (j>= * tailleAlloc) {b = 0;break;}	      
		from[j] = i+1;
		pascommececaracter = 0;
	      }
	  }
	else
	  if(!pascommececaracter)
	    {
	      to[j] = i;
	      pascommececaracter = 1;
	      j++;    
	    }
      }
    
    /* if last letters were in lower case */
    /*  printf("|%c|\n",seq[i-1]);
	printf(">%s<\n",seq); */
    if(seq[i-1] == (**caracter))
      {
	to[j] = i;
	j++;
      }

    /* from = (int *) realloc(from, (j+1)* sizeof(int));
       to = (int *) realloc(to, (j+1)* sizeof(int)); */
    
    * tailleAlloc = j;
    return(b);
  }

}

==> GeneR_2.10.0.tar.gz: GeneR/src/masked.cc <==
/*! \file masked.cc
 *  \brief functions to look for masked positions. Masked by letter or
 *  by lower / upper case.
 *  \version 1
 *  \date Created       : 26/10/05 
 *  \date Last Modified : Time-stamp: <2007-10-12 18:37:59 antoine>
 *  \author O. Rogier and A. Lucas
 *  \note Licence: CeCIll
 */


#include <stdio.h>
#include <string>
#include "GeneR_glob.h"
#include "masked.h"

#define min( A , B )  ( ( A ) < ( B ) ? ( A ) : ( B ) )



extern "C"
{
  void mask (int * bufno,int * from, int * to, int * n, char ** lettre,int * err);
  void maskee(char ** file, int * debSeq, int * tailleSeq, int * from, 
	      int * to,int * tailleAlloc, int* fromSeq, int* toSeq,int * err);
  void mask_buffer(int * bufno,  int * from, int * to,int * tailleAlloc,  
		   int* fromSeq, int* toSeq,int * err);
  void mask_buffer_char(int * bufno,  int * from, int * to,int * tailleAlloc, 
			char ** caracter,  int* fromSeq, int* toSeq, 
			int * err);
}


/** \fn mask (int * bufno,int * from, int * to, char ** lettre,int * err)
 * \brief mask parts of a sequence with letter 
 * \author A. Lucas
 */
void mask (int * bufno,int * from, int * to, int * n,char ** lettre,int * err)
{
  char * seq;
  int i,j;
  seq = GeneR_glob::instance()->buffer(bufno[0],0);
  if(seq == NULL)
    {
      *err = -1;
      return;
    }

  GeneR_glob::instance()->freeCompSeq(bufno[0]);


  for(i = 0; i< (*n); i++)
    {
      for(j=from[i];j<=to[i]; j++)
	seq[j-1] = lettre[0][0];

    }  

}

/** \fn maskee(char ** file, int * debSeq, int * tailleSeq, int * from, int * to,int * tailleAlloc, int * err)
 * \brief This function gives the masked sequences positions from a fasta file
 *  It returns : 1 if the vector of positions has been created
 *              0 if the allocation length is too small
 *             -1 if error (can't open the file) 
 * \author O. Orgier
 */
void maskee(char ** file, int * debSeq, int * tailleSeq, int * from, int * to,int * tailleAlloc,  int* fromSeq, int* toSeq,int * err)
{

  char * seq;
  FILE * fichier;

  
  fichier = fopen(*file,"r");
  if (fichier == NULL)
    {
      printf("File Name incorrect !\n"); 
      * err = -1;
      return;
    }
  seq = (char *) malloc ((* tailleSeq) * sizeof(char));
 


  *tailleSeq = masked::lecturefasta(fichier,seq,* debSeq,* tailleSeq)-1;

  if(*toSeq == 0)
    *toSeq = *tailleSeq;

  /*  * tailleSeq = strlen(seq)-1; */

  /*  seq[* tailleSeq] = '\0' ;*/
  /*printf("Sequence length : %ld \n",* tailleSeq);
    printf("Sequence :%s \n",seq); */

  * err = masked::codage(seq,tailleAlloc,from,to ,fromSeq, toSeq);
  /*if (*err == 0 ) {printf("Warning message : not all the sequence was masked.\n");}
    printf("Number of masked sequences: %d\n",*tailleAlloc);*/

  free(seq);
  return;
}


/** \brief This function gives the masked sequences positions
 * between pos begSeq and EndSeq. position are returned in vector
 * from and to.
 * It returns : 1 if the vector of positions has been created
 *              0 if the allocation length is too small
 *             -1 if error (can't open the file) 
 * \author A. Lucas and O. Rogier
 */
void mask_buffer(int * bufno,  int * from, int * to,int * tailleAlloc,
		 int * begSeq, int * endSeq,
		 int * err)
{

  char * seq;

  seq = GeneR_glob::instance()->buffer(bufno[0],0);
  if(seq == NULL)
    {
      *err = -1;
      return;
    }

  * err = masked::codage(seq,tailleAlloc,from,to,begSeq,endSeq);
  /*  if (*err == 0 ) {printf("Warning message : not all the sequence was masked.\n");}*/
  /*  printf("Number of masked sequences: %d\n",*tailleAlloc);*/
  
  
  return;
}



/** mask_buffer_char
 *
 *
 * \author A. Lucas and O. Rogier
 */
void mask_buffer_char(int * bufno,  int * from, int * to,int * tailleAlloc, 
		     char ** caracter, int* begSeq, int* endSeq, int * err)
{

  char * seq;


  seq = GeneR_glob::instance()->buffer(bufno[0],0);

  * err = masked::codage_char(seq,tailleAlloc,from,to,caracter,begSeq,endSeq);
  /*  if (*err == 0 ) {printf("Warning message : not all the sequence was masked.\n");}*/
  /*  printf("Number of masked sequences: %d\n",*tailleAlloc);*/
  
  
  return;
}


namespace masked{
  /** \fn lecturefasta (FILE * fichier,char * seq, int debSeq, int tailleSeq)
  * \brief Read a fasta file to a char*
   * \author A. Lucas and O. Rogier
   */ 
  int lecturefasta (FILE * fichier,char * seq, int debSeq, int tailleSeq)
  {
    int indice,longueur_cour,longueur_ligne,longueur_rest;
    char  tmp [512];

#ifndef __MINGW_H  
    if(fseek( fichier, debSeq, SEEK_SET))
      return(-1);
#else
    int i;
    for(i=0; i< debSeq; i++)
      if(fgetc(fichier) == EOF)
	return(-1);
#endif
    fgets(seq,tailleSeq,fichier);      
    indice =strlen(seq);
    seq[indice-1] = '\0'; 
    longueur_ligne = longueur_cour = indice-1;
    longueur_rest = tailleSeq - longueur_ligne;
  

    while(  fgets(tmp,min(512,longueur_rest),fichier) != NULL)
      {
	indice =strlen(tmp);
	tmp[indice-1] = '\0'; /* Delete \n */
	
	// memcpy: bien plus rapide que strcat 
	if(tmp[0] != '>')
	  memcpy(seq+longueur_cour,tmp,indice);
	longueur_cour += indice-1;
	longueur_rest -= indice-1;
	//strcat(seq,tmp);
      }

    /*  printf("seq: >%s<\n",seq); */

    fclose(fichier);
    return(longueur_cour);
  }


  /** codage 
   * \brief get positions when sequence change from lower to upper case 
   * between pos begSeq & endSeq
   * \author O. Rogier, modified by A. Lucas.
   */
  int codage(char * seq, int * tailleAlloc, int * from, int * to,
	     int * begSeq, int * endSeq)
    
  {
    int i,j,b,majuscule;

    j=0;
    b=1;
    majuscule = 1;

    for(i=begSeq[0]; i<endSeq[0] ; i++)
      {
	if(seq[i] > 'Z')
	  {
	    if(majuscule)
	      {
		if (j>= * tailleAlloc) {b = 0;break;}	      
		from[j] = i+1;
		majuscule = 0;
	      }
	  }
	else
	  if(!majuscule)
	    {
	      to[j] = i;
	      majuscule = 1;
	      j++;    
	    }
      }

    /* if last letters were in lower case */
    /*  printf("|%c|\n",seq[i-1]);
	printf(">%s<\n",seq); */
    if(seq[i-1] > 'Z') /* equivalent to if(majucule == 0)*/
      {
	to[j] = i;
	j++;
      }

    /* from = (int *) realloc(from, (j+1)* sizeof(int));
       to = (int *) realloc(to, (j+1)* sizeof(int)); */
  
    * tailleAlloc = j;
    return(b);
  }


  /** codage_char 
   * \brief get positions when sequence is a special char (by example N)
   * \author A. Lucas and O. Rogier 
   */
  int codage_char(char * seq, int * tailleAlloc, int * from, int * to,char ** caracter, int * fromSeq, int * endSeq)
    
  {
    int i,j,b,pascommececaracter;
    j=0;
    b=1;
    pascommececaracter = 1;
    
    for(i=fromSeq[0]; i < endSeq[0] ; i++)
      {
	if(seq[i] == (**caracter) )
	  {
	    if(pascommececaracter)
	      {
		if (j>= * tailleAlloc) {b = 0;break;}	      
		from[j] = i+1;
		pascommececaracter = 0;
	      }
	  }
	else
	  if(!pascommececaracter)
	    {
	      to[j] = i;
	      pascommececaracter = 1;
	      j++;    
	    }
      }
    
    /* if last letters were in lower case */
    /*  printf("|%c|\n",seq[i-1]);
	printf(">%s<\n",seq); */
    if(seq[i-1] == (**caracter))
      {
	to[j] = i;
	j++;
      }

    /* from = (int *) realloc(from, (j+1)* sizeof(int));
       to = (int *) realloc(to, (j+1)* sizeof(int)); */
    
    * tailleAlloc = j;
    return(b);
  }

}
